#!/bin/sh

readonly THIS_PATH="$(readlink -e "$(dirname "$0")")"

. "${THIS_PATH}/upgrade-state.inc"
. "${THIS_PATH}/offline-event.inc"

readonly WDSERVER_BIN='/opt/kaspersky/kesl/libexec/wdserver'
readonly PRODUCT_NAME='kesl'
readonly PRODUCT_VERSION='10.1.0.5062'
readonly PRODUCT_BIN='/opt/kaspersky/kesl/libexec/kesl'
readonly WDSERVER_LOG='/var/log/kaspersky/kesl/kesl_wdserver.log'
readonly WDSERVER_PID='/var/run/wdserver.pid'
readonly LAUNCHER_PID='/var/run/kesl_launcher.pid'
readonly LAUNCHER_LOG='/var/log/kaspersky/kesl/kesl_launcher.log'
readonly LAUNCHER_FILEPATH='/opt/kaspersky/kesl/libexec/kesl_launcher.sh'
readonly PRODUCT_CONTROL_BIN='/opt/kaspersky/kesl/bin/kesl-control'
readonly VAR_DIRECTORY='/var/opt/kaspersky/kesl/'
readonly SHARED_INIT_STORAGE_DIRECTORY='/opt/kaspersky/kesl/shared/init/storage/'
readonly VAR_STORAGE_DIRECTORY='/var/opt/kaspersky/kesl/private/storage/'
readonly SC_KLNAGENT_BIN='/opt/kaspersky/klnagent64/sbin/klnagent'
readonly READY_MARKER_FILEPATH='/var/run/kesl.mrk'
readonly INI_FILEPATH='/var/opt/kaspersky/kesl/common/kesl.ini'
readonly MONITOR_OAS_VERSION_FILEPATH='/proc/kav4fs_oas.version'
readonly INSTALL_FILEPATH='/opt/kaspersky/kesl/libexec/install.sh'
readonly INSTALL_LOG_FILEPATH='/var/opt/kaspersky/kesl/private/install.sh.log'
readonly UNINSTALL_FILEPATH='/opt/kaspersky/kesl/libexec/uninstall.sh'
readonly CURRENT='/var/opt/kaspersky/kesl/install-current'
readonly UPGRADE='/var/opt/kaspersky/kesl/install-upgrade'
readonly PREVIOUS='/var/opt/kaspersky/kesl/install-previous'
readonly FAILED='/var/opt/kaspersky/kesl/install-failed'
readonly CURRENT_NAME='install-current'
readonly UPGRADE_NAME='install-upgrade'
readonly PREVIOUS_NAME='install-previous'
readonly FAILED_NAME='install-failed'
readonly CRYPTO_SSL_FILEPATH='/opt/kaspersky/kesl/lib64/libcrypto_ssl.so'
readonly CRYPTO_SSL_SYMLINK_PREV_FILEPATH='/opt/kaspersky/kesl/lib64/libcrypto_ssl_prev'
readonly CRYPTO_SSL_SYMLINK1_FILEPATH='/opt/kaspersky/kesl/lib64/libcrypto_ssl_1_1.so'
readonly CRYPTO_SSL_SYMLINK2_FILEPATH='/opt/kaspersky/kesl/lib64/libcrypto_ssl_1_1.so.1.1'
readonly PREPARE_MODULE_SCRIPT_FILEPATH='/opt/kaspersky/kesl/libexec/prepare_module.sh'
readonly BACKUP_DIRECTORY="/var/opt/kaspersky/kesl/private/storage-backup"
readonly BACKUP_CURRENT_LNK="/var/opt/kaspersky/kesl/private/storage-backup.current"
readonly DEFAULT_FILE_MODE_CREATION_MASK='0177'

readonly LOGDATEFMT='+%F %T.%3N%:z'

readonly LINUX_KERNEL="$(uname -r)"

readonly PRODUCT_START_DELAY=180

readonly SERVER_REBOOT_DELAY=0

readonly WDRESTART=5

readonly WDSERVER_LOG_LEVEL=600

readonly WDSERVER_SOCK='/var/run/wdserver.sock'

readonly WDTERM_SIGNAL=2

readonly WDSERVER_MAX_MEM='90%'
readonly WDSERVER_MAX_SWAP='100%'

readonly WDSERVER_FLAGS="\
--trace-level ${WDSERVER_LOG_LEVEL} \
--trace-file ${WDSERVER_LOG} \
--n-restart ${WDRESTART} \
--master-timeout ${PRODUCT_START_DELAY} \
--reboot-timeout ${SERVER_REBOOT_DELAY} \
--wdsock ${WDSERVER_SOCK} \
--max-used-mem ${WDSERVER_MAX_MEM} \
--max-used-swap ${WDSERVER_MAX_SWAP} \
--term-signal ${WDTERM_SIGNAL} \
-p ${WDSERVER_PID} \
-d"

BACKUP_CURRENT=''
UPGRADE_STATE=''
PATCH_VERSION=''
BACKUP_NEW_UNIQUE_DIR=''

Die()
{
    local res=255
    if [ "$1" -eq "$1" ] 2>/dev/null; then
        res=$1
        shift
    fi

    {
        echo
        /bin/echo -e "$(basename "$0"): ERROR: $@"
        echo
    } >&2

    exit ${res}
}

Repeat()
{
    local n="$1"
    local ch="$2"
    printf "%${n}s\n" | tr " " "${ch}"
}

Log()
{
    /bin/echo -e "[$(date "${LOGDATEFMT}")] $@"
}

Trace()
{
    Log "$@"
}

GetBackupDir()
{
    local outVar="${1:?}"

    if [ ! -L "${BACKUP_CURRENT_LNK}" ]; then
        return 0
    fi

    local current
    current="$(readlink -n "${BACKUP_CURRENT_LNK}")"

    local rc
    if [ "${current}" = "${SHARED_INIT_STORAGE_DIRECTORY}" ]; then
        rc=1
    elif [ -z "${current}" ] || [ ! -d "${current}" ]; then
        rc=1
        current="${SHARED_INIT_STORAGE_DIRECTORY}"
    else
        rc=2
    fi

    eval "${outVar}='${current}'"

    return ${rc}
}

MakeUniqueDir()
{
    Trace "MakeUniqueDir()"


    if [ ! -d "${BACKUP_DIRECTORY}" ]; then
        mkdir -v "${BACKUP_DIRECTORY}"
    fi

    local tmp
    tmp="$(mktemp -d --tmpdir="${BACKUP_DIRECTORY}" "$(date '+%FT%T%:z') (XXXXXX)")"
    local res=$?

    if [ $res -ne 0 ] || [ -z "${tmp}" ]; then
        Log "Cannot create temporary directory in dir '${BACKUP_DIRECTORY}'"
    else
        BACKUP_NEW_UNIQUE_DIR="${tmp}"
    fi

    return $res
}

RemoveNewUniqDir()
{
    Trace "RemoveNewUniqDir()"

    local path="${BACKUP_NEW_UNIQUE_DIR}"

    Log "Removing tmp directory '${path}'"

    if [ ! -z "${path}" ]; then
        rm -rfv "${path}"
    fi

    BACKUP_NEW_UNIQUE_DIR=""
}

ClearOldBackupDirs()
{
    Trace "ClearOldBackupDirs()"


    local currentName
    currentName="$(basename "${BACKUP_NEW_UNIQUE_DIR}")" || currentName=""
    find "${BACKUP_DIRECTORY}" -mindepth 1 -maxdepth 1 -not -name "${currentName}" \
        -exec rm -rfv {} \;
}

PrepareBackup()
{
    Trace "PrepareBackup()"

    if MakeUniqueDir; then
        Log "Preparing backup at '${BACKUP_NEW_UNIQUE_DIR}' directory ..."

        cp -frpv -t "${BACKUP_NEW_UNIQUE_DIR}" "${VAR_STORAGE_DIRECTORY}/."
        local res=$?
        if [ ${res} -ne 0 ]; then
            Log "Preparing backup completed with error: ${res}"
            RemoveNewUniqDir
            return ${res}
        fi

        Log "Preparing backup successfully done"
    fi
}

CommitBackup()
{
    Trace "CommitBackup()"

    local res

    if [ -z "${BACKUP_NEW_UNIQUE_DIR}" ]; then
        Log "Commit backup at init dir '${SHARED_INIT_STORAGE_DIRECTORY}' ..."
        ln -Tsfv "${SHARED_INIT_STORAGE_DIRECTORY}" "${BACKUP_CURRENT_LNK}"
        res=$?
    else
        Log "Commit backup at dir '${BACKUP_NEW_UNIQUE_DIR}' ..."
        ln -Tsfv "${BACKUP_NEW_UNIQUE_DIR}" "${BACKUP_CURRENT_LNK}"
        res=$?
        if [ ${res} -ne 0 ]; then
            RemoveNewUniqDir
        else
            ClearOldBackupDirs
            BACKUP_NEW_UNIQUE_DIR=""
        fi
    fi

    if [ ${res} -ne 0 ]; then
        Log "Commit backup error: cannot create 'current' symlink: ${res}"
    else
        Log "Commit backup OK"
    fi

    return ${res}
}

RestoreCryptoSSL()
{
    Trace "RestoreCryptoSSL()"

    local origin="${CRYPTO_SSL_FILEPATH}"
    local prev="${CRYPTO_SSL_SYMLINK_PREV_FILEPATH}"
    local s1="${CRYPTO_SSL_SYMLINK1_FILEPATH}"
    local s2="${CRYPTO_SSL_SYMLINK2_FILEPATH}"

    local path="${origin}"

    if [ -e "${prev}" ]; then
        local prevPath
        prevPath="$(readlink -e "${prev}")"

        if [ -f "${prevPath}" ]; then
            path="${prevPath}"
        fi
    fi

    ln -sfT "${path}" "${s1}"
    ln -sfT "${path}" "${s2}"

    unlink "${prev}"

    return 0
}

RestoreFromBackup()
{
    Trace "RestoreFromBackup()"

    local current="$1"

    if [ "${current}" = "" ]; then
        current="${SHARED_INIT_STORAGE_DIRECTORY}"
    fi

    Log "Restoring from backup at '${current}'"

    rm -rfv "${VAR_STORAGE_DIRECTORY}/"*
    cp -frpv -t "${VAR_STORAGE_DIRECTORY}" "${current}/."

    RestoreCryptoSSL

    Log "Restoring done"

    return 0
}

IsProductStarting()
{

    local prodPidFile='/var/run/kesl.pid'
    local prodBin='/opt/kaspersky/kesl/libexec/kesl'

    local prodNb
    prodNb="$(pidof "${prodBin}" | wc -w)"

    if [ "${prodNb}" -ne 1 ]; then
        return 1
    fi

    local pid
    if [ -n "${prodPidFile}" -a -f "${prodPidFile}" ]; then
        pid="$(head -n 1 "${prodPidFile}" 2>/dev/null)"
        if [ -z "${pid}" ]; then
            return 1
        fi
        ps -p "${pid}" --context | grep "${prodBin}" >/dev/null 2>&1
        return $?
    else
        return 1
    fi
}

IsProductReady()
{
    IsProductStarting && test -f "${READY_MARKER_FILEPATH}"
}

Stop()
{
    Trace "Stop()"

    local wdPid
    wdPid="$(head "${WDSERVER_PID}")"
    if [ ! -z "${wdPid}" ]; then
        while IsProductStarting; do
            Trace "Product is still running: try to kill the watchdog by SIGINT: wdPid="${wdPid}""
            kill -INT "${wdPid}"
            sleep 1
        done

        Trace "kill -9 for wdPid=${wdPid}"
        kill -9 "${wdPid}" > /dev/null 2>&1
        Trace "wait for wdPid=${wdPid}"
        wait "${wdPid}" > /dev/null 2>&1
        Trace "wait done"
    fi

    Trace "Stop() done"

    exit 0
}

ReadIni()
{
    local fileName="$1"
    local itemName="$2"
    local outVar="$3"

    if [ ! -f "${fileName}" ]; then
        return 1
    fi

    local value
    if ! value=$(grep "^\s*${itemName}\s*=" "${fileName}") 2>/dev/null; then
        return 2
    fi

    if ! value=$(echo "${value}" | sed -e 's:\(.*=\s*\)\(\S*\)\(\s*\):\2:'); then
        return 3
    fi

    eval "${outVar}='${value}'"
}

IsFanotifyOn()
{
    local useNotify
    if ! ReadIni "${INI_FILEPATH}" 'UseFanotify' useNotify; then
        return 1
    fi

    if ! echo "${useNotify}" | grep -iE '^(true|yes|1)$' >/dev/null; then
        return 1
    fi

    return 0
}

IsKernelModuleLoaded()
{
    test -f "${MONITOR_OAS_VERSION_FILEPATH}"
    return $?
}

PrepareKernelModule()
{
    Trace "PrepareKernelModule(): $@"

    local atPath="${1:?}"
    shift

    if ! "${atPath}/${PREPARE_MODULE_SCRIPT_FILEPATH}" "$@"; then
        return 1
    fi

    return 0
}

RemoveKernelModuleFiles()
{
    rm -f "/lib/modules/${LINUX_KERNEL}/misc/kav4fs_oas.ko"
    rm -f "/lib/modules/${LINUX_KERNEL}/misc/redirfs.ko"
}

CopySymlink()
{
    cp -TP "$1" "$2" || Die "'cp' failed CopySymlink '$1' '$2' (exit code=$?)"
}

MoveSymlink()
{
    mv -fT "$1" "$2" || Die "'mv' failed MoveSymlink '$1' '$2' (exit code=$?)"
}

RemoveSymlink()
{
    rm -f "$1"
}

ReloadKlnagent()
{
    if [ -x "${SC_KLNAGENT_BIN}" ]; then
        "${SC_KLNAGENT_BIN}" -reload-connectors
    fi
}

SwitchToUpgradeState()
{
    local newState="$1"
    shift

    Log "Switch upgrade state: '${UPGRADE_STATE}' -> '${newState}': $@"

    "$@"

    GetUpgradeState UPGRADE_STATE PATCH_VERSION
    if [ "${UPGRADE_STATE}" != "${newState}" ]; then
        Log "WARNING: GetUpgradeState() returned unexpected upgrade state: ${UPGRADE_STATE}; expected: ${newState}"
    fi
}

ExecLauncher()
{
    local ver="$1"

    Trace "ExecLauncher($1)"

    exec "${CURRENT}/${LAUNCHER_FILEPATH}" || Die "Failed to exec ${ver} launcher (exit code=$?)"
}

DoUninstall()
{
    Trace "DoUninstall()"

    "${CURRENT}/${UNINSTALL_FILEPATH}"
}

DoUpgrade()
{
    Trace "DoUpgrade()"

    if ! IsFanotifyOn; then
        RemoveKernelModuleFiles
    fi

    "${UPGRADE}/${INSTALL_FILEPATH}" "${UPGRADE}/${INSTALL_LOG_FILEPATH}" || return $?

    ReloadKlnagent

    return 0
}

DoStart()
{
    Trace "DoStart()"

    Log "Try to start version '${PRODUCT_VERSION}' at install dir '$(readlink ${CURRENT})'"
}

DoUpgradeFailed()
{
    Trace "DoUpgradeFailed()"

    "${FAILED}/${UNINSTALL_FILEPATH}"
}

DoStartFailed()
{
    Trace "DoStartFailed()"

    "${FAILED}/${UNINSTALL_FILEPATH}"
}

DoRollingBack()
{
    Trace "DoRollingBack()"

    if ! IsFanotifyOn; then
        RemoveKernelModuleFiles
    fi

    if ! "${CURRENT}/${INSTALL_FILEPATH}" "${CURRENT}/${INSTALL_LOG_FILEPATH}"; then
        EmitOfflineEvent 'PatchRollbackError' 'Error' 'PatchVersion' "${PATCH_VERSION}" 'Reason' ''
    fi

    ReloadKlnagent
}

AssumeInstalled()
{
    Trace "AssumeInstalled()"

    local prevDirName="$(readlink "${PREVIOUS}")"

    SwitchToUpgradeState 'Installed' RemoveSymlink "${PREVIOUS}"

    Log "Remove previous version files at '${VAR_DIRECTORY}${prevDirName}'"
    rm -rf "${VAR_DIRECTORY}${prevDirName}"
}

ProcessUpgradeState()
{
    case "${UPGRADE_STATE}" in
        'Installed')
            return 1
            ;;
        'Uninstall')
            if ! IsFanotifyOn; then
                if IsKernelModuleLoaded; then
                    Log "Cannot do 'Uninstall' now: UseFanotify=false and kernel module is loaded: reboot must be done"
                    return 1
                fi
                if ! PrepareKernelModule "${UPGRADE}" '--build-only'; then
                    Log "Failed to build new kernel module"
                    SwitchToUpgradeState 'RollingBack' MoveSymlink "${UPGRADE}" "${FAILED}"
                    EmitOfflineEvent 'PatchInstallError' 'Error' 'PatchVersion' "${PATCH_VERSION}" 'Reason' 'Failed to build new kernel module'
                    return 0
                fi
            fi

            DoUninstall
            SwitchToUpgradeState 'Upgrade' CopySymlink "${CURRENT}" "${PREVIOUS}"
            ;;
        'Upgrade')
            if DoUpgrade; then
                SwitchToUpgradeState 'Start' MoveSymlink "${UPGRADE}" "${CURRENT}"
                ExecLauncher 'new'
            else
                SwitchToUpgradeState 'UpgradeFailed' MoveSymlink "${UPGRADE}" "${FAILED}"
            fi
            ;;
        'Start')
            DoStart
            return 1
            ;;
        'UpgradeFailed')
            DoUpgradeFailed
            SwitchToUpgradeState 'RollingBack' RemoveSymlink "${PREVIOUS}"
            EmitOfflineEvent 'PatchInstallError' 'Error' 'PatchVersion' "${PATCH_VERSION}" 'Reason' 'Upgrade procedure failed'
            ;;
        'StartFailed')
            DoStartFailed
            SwitchToUpgradeState 'RollingBack' MoveSymlink "${PREVIOUS}" "${CURRENT}"
            EmitOfflineEvent 'PatchInstallError' 'Error' 'PatchVersion' "${PATCH_VERSION}" 'Reason' 'Failed to start application'
            ExecLauncher 'old'
            ;;
        'RollingBack')
            DoRollingBack
            SwitchToUpgradeState 'RolledBack' CopySymlink "${FAILED}" "${UPGRADE}"
            EmitOfflineEvent 'PatchRollbackCompleted' 'Info' 'PatchVersion' "${PATCH_VERSION}"
            ;;
        *)
            Log "Nothing to do at upgrade state '${UPGRADE_STATE}'"
            return 1
            ;;
    esac

    return 0
}

Main()
{
    Repeat 85 '='
    Trace "Main(): pid=$$, PPID=$PPID"

    trap "" HUP
    trap "" INT
    trap "Stop" TERM

    echo $$ > "${LAUNCHER_PID}"

    while :; do
        GetUpgradeState UPGRADE_STATE PATCH_VERSION || Die "Failed to get upgrade state (exit code=$?): returned state='${UPGRADE_STATE}'"
        Log "Current upgrade state: '${UPGRADE_STATE}'"
        if ! ProcessUpgradeState; then
            break
        fi
    done

    GetBackupDir BACKUP_CURRENT
    local backupState=$?

    local backupPrepared=0
    if [ ${backupState} -ne 0 ]; then
        PrepareBackup
        backupPrepared=$?
    fi

    local wdPid
    local startDetectedFl
    local alreadyRestoredFl=0

    while true; do
        startDetectedFl=0

        Log "Running wdserver ..."

        "${WDSERVER_BIN}" ${WDSERVER_FLAGS} -e "${PRODUCT_BIN}"

        while :; do
            sleep 5 &
            wait

            wdPid="$(pidof "${WDSERVER_BIN}")"

            if [ -z "${wdPid}" ]; then
                Log "'wdserver' process is not found"
                Log "Print product process's state:\n$(ps -ef | grep "${PRODUCT_NAME}")"
                break
            fi

            if IsProductReady; then
                if [ ${startDetectedFl} -eq 0 ]; then
                    startDetectedFl=1

                    if [ "${UPGRADE_STATE}" = 'Start' ]; then
                        AssumeInstalled
                    fi

                    if [ "${backupPrepared}" -eq 0 ]; then
                        backupPrepared=1
                        if CommitBackup; then
                            GetBackupDir BACKUP_CURRENT
                            backupState=$?
                            Log "Backup has been switched to '${BACKUP_CURRENT}' directory (state ${backupState})"
                        fi
                    fi
                fi
            fi
        done

        if [ "${backupPrepared}" -eq 0 ]; then
            if [ ${backupState} -ne 0 ]; then
                Log "Clear prepared but not committed backup"
                backupPrepared=1
                RemoveNewUniqDir
            fi
        fi

        if [ ${startDetectedFl} -eq 0 ] && [ "${UPGRADE_STATE}" = 'Start' ]; then
            Log "******* Failed to start product the first time after upgrade *******"
            SwitchToUpgradeState 'StartFailed' CopySymlink "${CURRENT}" "${FAILED}"
            ProcessUpgradeState
        fi

        if [ ${startDetectedFl} -eq 0 ] && [ ${alreadyRestoredFl} -ne 0 ]; then
            if [ ${backupState} -eq 2 ]; then
                Log "Switch backup to init storage (state 1)"
                BACKUP_CURRENT="${SHARED_INIT_STORAGE_DIRECTORY}"
                backupState=1
                Log "Setting current symlink to '${SHARED_INIT_STORAGE_DIRECTORY}'"
                ln -Tsfv "${SHARED_INIT_STORAGE_DIRECTORY}" "${BACKUP_CURRENT_LNK}"
            else
                if [ ${backupState} -eq 1 ]; then
                    Log "Switch backup to empty (state 0)"
                    BACKUP_CURRENT=''
                    backupState=0
                    Log "unlink current symlink"
                    unlink "${BACKUP_CURRENT_LNK}"
                fi
                Die "Product cannot start with the init config"
            fi
        fi

        RestoreFromBackup "${BACKUP_CURRENT}"
        alreadyRestoredFl=1


    done
}

umask "${DEFAULT_FILE_MODE_CREATION_MASK}" && Main "$@" >>"${LAUNCHER_LOG}" 2>&1
