package KL::AppInfo;

use strict;
use KL::Utils;
use File::Spec;
use POSIX qw(setlocale);

use constant
{
    LFS_TASK_ID_UNDEFINED         => 0,
    LFS_TASK_ID_OAS               => 1,
    LFS_TASK_ID_ODS               => 2,
    LFS_TASK_ID_SCANFILE          => 3,
    LFS_TASK_ID_BOOTSCAN          => 4,
    LFS_TASK_ID_MEMORYSCAN        => 5,
    LFS_TASK_ID_UPDATE            => 6,
    LFS_TASK_ID_ROLLBACK          => 7,
    LFS_TASK_ID_RETRANSLATE       => 8,
    LFS_TASK_ID_LICENSE           => 9,
    LFS_TASK_ID_BACKUP            => 10,
    LFS_TASK_ID_INTEGRITY_MONITOR => 11,
    LFS_TASK_ID_FIREWALL          => 12,
    LFS_TASK_ID_ANTI_CRYPTOR      => 13,
    LFS_TASK_ID_USERTASKMIN       => 100
};

BEGIN
{
    my $lang = substr(setlocale("LC_ALL"), 0, 2);
    my @LFS_LANGS = split(/\s*,\s*/, "en,ru,de,fr");
    if (grep {$_ eq $lang} @LFS_LANGS)
    {
        eval "require KL::Localization::${lang}_title; KL::Localization::${lang}_title->import();";
    }
    else
    {
        require KL::Localization::en_title;
        KL::Localization::en_title->import();
    }
}

sub new
{
    my $class = shift;
    my $values = {};

    $values = read_app_info();

    my $self = { VALUES => $values };

    bless $self, $class;
}

sub value
{
    my $self = shift;
    my $section = shift;
    my $name = shift;

    return $self->{VALUES}->{$section}->{$name};
}

sub read_app_info
{
    my $filename = "/var/opt/kaspersky/kesl/common/kesl.ini";

    my $result =  KL::Utils::load_ini_file($filename);

	if (!exists $result->{General}) {
		$result->{General} = {};
	}
	if (!exists $result->{General}->{LFS_DIRECTORY}) {
		$result->{General}->{LFS_DIRECTORY} = "/opt/kaspersky/kesl/";
	}
	my $opt_directory = "/opt/kaspersky/kesl/";
	my $var_directory = "/var/opt/kaspersky/kesl/";
    my $var_private_directory = "/var/opt/kaspersky/kesl/private/";
	my $log_directory = "/var/log/kaspersky/kesl/";
	my $etc_directory = "/etc/opt/kaspersky/kesl/";
	$result->{'paths'} = {};
	$result->{'paths'}{'.ini'} = $filename;
	if (exists $result->{'paths'}{'var'}) {
		$var_directory = $result->{'paths'}{'var'};
	} else {
		$result->{'paths'}{'var'} = $var_directory;
	}
	if (exists $result->{'paths'}{'opt'}) {
		$opt_directory = $result->{'paths'}{'opt'};
	} else {
		$result->{'paths'}{'opt'} = $opt_directory;
	}
	if (exists $result->{'paths'}{'log'}) {
		$log_directory = $result->{'paths'}{'log'};
	} else {
		$result->{'paths'}{'log'} = $log_directory;
	}
	if (exists $result->{'paths'}{'etc'}) {
		$etc_directory = $result->{'paths'}{'etc'};
	} else {
		$result->{'paths'}{'etc'} = $etc_directory;
	}
	my $shared_directory = File::Spec->catfile($opt_directory, "shared/");
	$result->{'paths'}{'docs'} = File::Spec->catfile($opt_directory, "doc/");
	$result->{'paths'}{'bins'} = File::Spec->catfile($opt_directory, "bin/");
	$result->{'paths'}{'shared'} = ${shared_directory};
	$result->{'paths'}{'srcs'} = File::Spec->catfile($opt_directory, "src/");
	$result->{'paths'}{'control'} = File::Spec->catfile($result->{'paths'}{'bins'}, "kesl-control");
	$result->{'paths'}{'setup'} = File::Spec->catfile($result->{'paths'}{'bins'}, "kesl-setup.pl");
	$result->{'paths'}{'agreed_eula'} = File::Spec->catfile($var_private_directory, "agreed_eula.txt");
	$result->{'paths'}{'license_template'} = File::Spec->catfile($result->{'paths'}{'docs'}, "license.%LANG%");
	$result->{'paths'}{'agreed_ksnua'} = File::Spec->catfile($var_private_directory, "agreed_ksnua.txt");
	$result->{'paths'}{'ksn_license_template'} = File::Spec->catfile($result->{'paths'}{'docs'}, "ksn_license.%LANG%");
	$result->{'paths'}{'init_d_script'} = "/etc/init.d/kesl-supervisor";
	$result->{'paths'}{'fanotify-checker'} = File::Spec->catfile($result->{'paths'}{'bins'}, "fanotify-checker");
    $result->{'product'}{'license_langs'} = "en,ru,de,fr";
	$result->{'product'}{'title'} = $MSG_LFS_PRODUCT_TITLE;
	$result->{'product'}{'version'} = "10.1.0.5062";
	$result->{'product'}{'release'} = "Release";
	return $result;
}

1;
