/** @file
@brief       ioctl API to kavmonitor driver

@copyright   (C) Kaspersky Lab 2004-2008
*/

#ifndef KERNEL__OAS__SMALL__DEVICE_H
#define KERNEL__OAS__SMALL__DEVICE_H

#ifdef __FreeBSD__
#include <sys/param.h>
#endif

#ifndef __KERNEL__
#include <sys/types.h>
#endif

typedef struct
{
    int cache_size;
} monitor_config_ioctl;

typedef struct
{
    int32_t cache_size;
    int32_t version;
    uint32_t max_fname_len;     /* maximum file name length, default is 16K */
    int8_t deny_too_long_fname; /* (bool) deny accsess if a file name length >
                                   max_fname_len */
} monitor_config2_ioctl;

typedef struct
{
    int pid;
    char* path;

} monitor_register_proc_ioctl;


/* monitor ioctl commands (for kernel module only) */

#if defined(__linux__)


#define MONITOR_REGISTER_CMD_OLD_COMPAT 0x10
#define MONITOR_GROUP_CMD_OLD_COMPAT 0x11
#define MONITOR_CONFIG_CMD_OLD_COMPAT 0x21

#define MONITOR_REGISTER_CMD_COMPAT _IO('K', 0x10)
#define MONITOR_GROUP_CMD_COMPAT _IO('K', 0x11)
#define MONITOR_CONFIG_CMD _IOW('K', 0x21, monitor_config_ioctl)
#define MONITOR_CONFIG2_CMD _IOW('K', 0x22, monitor_config2_ioctl)

#define MONITOR_REGISTER_FILE_CMD _IO('K', 0x50)
#define MONITOR_REGISTER_PROC_CMD _IO('K', 0x51)
#define MONITOR_PING_CMD _IO('K', 0x52)
#define MONITOR_CLEAR_CACHE_CMD _IO('K', 0x53)
#define MONITOR_START _IO('K', 0x54)

#else
#if __FreeBSD_version < 500000
#define MONITOR_REGISTER_CMD_COMPAT 0x10
#define MONITOR_GROUP_CMD_COMPAT 0x11
#else
#define MONITOR_REGISTER_CMD_COMPAT _IO('B', 0x10)
#define MONITOR_GROUP_CMD_COMPAT _IO('B', 0x11)
#endif
#define MONITOR_CONFIG_CMD _IOW('B', 0x21, monitor_config_ioctl)
#define MONITOR_CONFIG2_CMD _IOW('B', 0x22, monitor_config2_ioctl)
#define KAVMON_DEVICE_MAJOR 207
#define MONITOR_REGISTER_FILE_CMD _IO('B', 0x50)
#define MONITOR_REGISTER_PROC_CMD _IO('B', 0x51)
#define MONITOR_PING_CMD _IO('B', 0x52)
#define MONITOR_CLEAR_CACHE_CMD _IO('B', 0x53)
#define MONITOR_START _IO('B', 0x54)

#endif


#define KAVMON_DEVICE_COMPAT "/dev/kavmonitor"
#define KAVMON_DEVICE "/dev/kav4fs_oas"

#ifndef __KERNEL__
#define MINORBITS 8
#define MKDEV(ma, mi) (((ma) << MINORBITS) | (mi))
#endif

#ifdef __linux__
#ifdef __KERNEL__
//#define KAVMON_PROC_COMPAT	"kavmonitor"
#define KAVMON_PROC "kav4fs_oas"
#else
//#define KAVMON_PROC_COMPAT	"/proc/kavmonitor"
#define KAVMON_PROC "/proc/kav4fs_oas"
#endif
#else
#define KAVMON_PROC_COMPAT "/dev/kavinfo"
#define KAVMON_PROC "/dev/kavproc"
#endif

#endif // KERNEL__OAS__SMALL__DEVICE_H
